@@ -108,3 +108,7 @@ class DeploymentError(OctueSDKException):
 
 class CloudStorageBucketNotFound(OctueSDKException):
     """Raise if attempting to access a cloud storage bucket that cannot be found."""
+
+
+class PushSubscriptionCannotBePulled(OctueSDKException):
+    """Raise if attempting to pull a push subscription."""
